@@ -1,15 +1,16 @@
 import os.path
 import secrets
 
+from PIL import Image
+import cv2
 import pdfkit
 import requests
 import base64
-import json
 import datetime
-from copy import deepcopy
 import pickle
 import tempfile
 import functools
+import imgkit
 
 
 class Address:
@@ -120,7 +121,7 @@ class Delivery:
         self.shipment_id = req.json()["data"]["shipmentId"]
         self.consignment = req.json()["data"]["consignmentDetail"][0]["consignmentNumber"]
         self.parcels = [x for x in req.json()["data"]["consignmentDetail"][0]["parcelNumbers"]]
-        self.dpd = dpd
+        self._dpd = dpd
 
     def __str__(self):
         return f"<Delivery : consignment: {self.consignment} - weight: {self.parcel.weight}kg - pcs: {self.parcel.pcs} - name: {self.parcel.name}>"
@@ -187,9 +188,9 @@ class Delivery:
         :param out_type:  1= html, 2= citizen-clp, 3= eltron-epl
         :return: data
         """
-        return self.dpd._get_label(self, out_type)
+        return self._dpd._get_label(self, out_type)
 
-    def save_pdf_label(self, out_path, filename, margins=[20, 20, 20, 20], paper_size="A4", zoom=1):
+    def save_pdf_label(self, out_path=None, filename=None, margins=[20, 20, 20, 20], paper_size="A4", zoom=1):
         """
         generates a PDF version of the label, beware this is fairly slow. Around 1 second+
 
@@ -197,7 +198,7 @@ class Delivery:
         :param filename: the filename of the pdf. If none supplied then a random filename will be generated and returned.
         :param margins: top, right, bottom, left margins in mm. Can be used to tweak the position of the label on the page
         :param paper_size: default A4
-        :param zoom: default 1  - no zoom
+        :param zoom: default 1  - no zoom  NOT WORKING CURRENTLY
         :return: output path string
         """
 
@@ -210,21 +211,22 @@ class Delivery:
 
         # create filename if not found
         if not filename:
-            filename = secrets.token_urlsafe(10) + ".pdf"
+            filename = secrets.token_urlsafe(10)
 
         # get final path
         final_path = out_path + "/" if out_path[-1] != "/" else "" + filename
 
         # convert margins to inches
-        margins = [x * 254 for x in margins]
+        margins = [x / 25.4 for x in margins]
 
         # get wkhtmltopdf options
         options = {'page-size': paper_size,
                    'margin-top': f'{margins[0]}in',
-                   'margin-right': f'{margins[2]}in',
-                   'margin-bottom': f'{margins[3]}in',
-                   'margin-left': f'{margins[4]}in',
-                   '--quiet': ''
+                   'margin-right': f'{margins[1]}in',
+                   'margin-bottom': f'{margins[2]}in',
+                   'margin-left': f'{margins[3]}in',
+                   '--quiet': '',
+                   'zoom': zoom,
                    }
 
         css = """<style>
@@ -239,11 +241,76 @@ class Delivery:
         else:
             return final_path
 
+    def get_png_labels(self, out_type=1):
+        """
+        used to return labels in a list, can be PIL or Numpy output type.
+        This method does not SAVE the images
+
+        :param out_type: 1 = Numpy arrays 2 = Pil Images
+        :return: List of images
+        """
+
+        #get CSS for label
+
+        css = """<style>
+                   div{font-family: Arial, Helvetica, sans-serif;}
+                   p{font-family: Arial, Helvetica, sans-serif;}
+                   body{font-family: Arial, Helvetica, sans-serif;}
+                 </style>
+        """
+
+        #get temp file
+        temp = tempfile.gettempdir() + "/temp_label.png"
+        imgkit.from_string(css + self.get_label().decode(), temp, options={"quality": 4, "width": 400,'--quiet': ''})
+        #read image
+        label_image = cv2.imread(temp)
+
+        #set positions
+        dist = 400
+        images = []
+        endy = 408
+        starty = 11
+
+        for x in range(len(self.parcels)):
+            label = label_image[starty:endy, 19:404, :]
+            images.append(label if out_type == 1 else Image.fromarray(label))
+            endy += dist * 2
+            starty += dist * 2
+
+        return images
+
+    def save_png_labels(self, out_path=None, filename=None):
+        """
+        Used to save PNG label images. When specifying the filename do not put the extension, its inferred already
+        :param out_path: path to save images to. If none then will be the current working directory
+        :param filename: filename without the .png  i.e   mylabel
+        if a blank filename is passed random file name will be used
+        :return: list of filenames
+        """
+
+        #check out path and use relative if needed
+        if not out_path:
+            out_path = "./"
+
+        if not os.path.isdir(out_path):
+            raise Exception("Out path not valid")
+
+        # create filename if not found
+        if not filename:
+            filename = secrets.token_urlsafe(10) + ".pdf"
+
+        #get the final path
+        final_path = out_path + "/" if out_path[-1] != "/" else "" + filename
+
+        #get list for to reutrn
+        filenames = []
+        for i, img in enumerate(self.get_png_labels(2)):
+            filenames.append(f"{final_path}-{str(i)}.png")
+            img.save(filenames[-1])
+
+        return filenames
 
 class DPD:
-    """
-
-    """
 
     def __init__(self, user, password, account_no, sender_address, host="https://api.dpd.co.uk"):
         """
@@ -379,31 +446,7 @@ class DPD:
         return req.content
 
     @_get_check_geosession
-    def create_shipment(self, parcel, service):
-        """
-        Used to create a DPD shipment
-
-        :param parcel: Parcel object
-        :param service: DPD service code i.e 1^12
-        :return: Delivery Object
-
-        """
-        if not parcel or type(parcel) is not Parcel:
-            raise Exception("Parcel not found, please add a Parcel object")
-        if not service or type(service) is not str:
-            raise Exception("service not valid")
-
-        # get the DPD shipping object
-        ship_obj = self._get_ship_obj(parcel, service)
-        # get the request
-        req = requests.post(self.host + "/shipping/shipment", headers=self._get_header(), json=ship_obj)
-        # validate the request
-        self._check_response_valid(req)
-        # get the delivery object
-        Delivery(parcel, service)
-
-    @_get_check_geosession
-    def create_shipment(self, parcel, service):
+    def create_delivery(self, parcel, service):
 
         """
         Used to create a DPD shipment
@@ -457,9 +500,9 @@ class DPD:
                             "countryCode": "GB",
                             "postcode": self.sender_address.postcode,
                             "street": self.sender_address.address_1,
-                            "locality": self.sender_address.address_2 if not self.sender_address.address_3 else "",
-                            "town": self.sender_address.address_3 if self.sender_address.address_3 else self.sender_address.address_2,
-                            "county": self.sender_address.address_4 if self.sender_address.address_4 else self.sender_address.address_2,
+                            "locality": self.sender_address.address_2 if not self.sender_address.address_3 == "" else "",
+                            "town": self.sender_address.address_2 if self.sender_address.address_3 == "" else self.sender_address.address_3,
+                            "county": self.sender_address.address_4,
                         }
                     },
                     "deliveryDetails": {
@@ -472,8 +515,8 @@ class DPD:
                             "countryCode": "GB",
                             "postcode": parcel.postcode,
                             "street": parcel.address_1,
-                            "locality": parcel.address_2 if not parcel.address_3 else "",
-                            "town": parcel.address_3 if parcel.address_3 else parcel.address_2,
+                            "locality": parcel.address_2 if not parcel.address_3 == "" else "",
+                            "town": parcel.address_2 if parcel.address_3 == "" else parcel.address_3,
                             "county": parcel.address_4,
                         },
                         "notificationDetails": {
